#include <errno.h>
#include <stdio.h>
#include <string.h>

#include "vpass.h"
#include "vp_err.h"

static char	cvs_id[] __unused =
"$Id: vp_err.c,v 1.1 2001/08/21 12:33:16 roam Exp $";

static const char *errs[VP_ERR_LAST] = {
	"no error",
	"out of memory",
	"bad command-line arguments",
	"fgets",
	"network subsystem not initialized",
	"no UCSPI protocol specified",
	"network read error",
	"network write error",
	"communications protocol error",
	"INTERNAL ERROR"
};

static int errsn[VP_ERR_LAST] = {
	0, 0, 0,	/* NONE, NOMEM, CMDLINE */
	1,		/* FGETS */
	0,		/* NET_NOTINIT */
	0,		/* NET_UCS_NOPROTO */
	1, 1,		/* READ, WRITE */
	0,		/* COMM */
	0		/* INT */
};

static char	errbuf[512];

/*
 * Function:
 * 	vp_strerror			- build an error message string
 * Inputs:
 * 	code				- error code
 * Returns:
 * 	pointer to static error message string
 * Modifies:
 * 	errbuf
 */

char *
vp_strerror(vp_err_t code) {
	
	if (code > VP_ERR_INT)
		code = VP_ERR_INT;

	if (errsn[code])
		snprintf(errbuf, sizeof(errbuf), "%s: %s", errs[code],
		    strerror(errno));
	else
		snprintf(errbuf, sizeof(errbuf), "%s", errs[code]);
	
	return (errbuf);
}

/*
 * Function:
 * 	vp_prerror			- print error message and context
 * Inputs:
 * 	s				- context string to be printed
 * 	code				- error code
 * Returns:
 * 	code
 * Modifies:
 * 	nothing; writes to stderr
 */

vp_err_t
vp_prerror(const char *s, vp_err_t code) {
	
	if (code > VP_ERR_INT)
		code = VP_ERR_INT;
	
	fprintf(stderr, "vpass error: %s: %s\n", s, vp_strerror(code));
	return (code);
}

