#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "vpass.h"
#include "vp_net.h"
#include "vp_net_ucspi.h"
#include "vp_util.h"

static char	cvs_id[] __unused =
"$Id: vp_net.c,v 1.2 2001/08/21 13:05:34 roam Exp $";

vp_nmode	vp_net_mode;

static vp_err_t	vp_net_sendexp(int fds[2], const char *s, char **resp,
		int *perr);
static vp_err_t	vp_net_cmdresp2(int fds[2], const char *cmd, const char *arg);

vp_err_t
vp_net_connect(int fds[2]) {

	switch (vp_net_mode) {
		case VP_NET_UCSPI:
			return (vp_net_ucspi_connect(fds));

		default:
			return (VP_ERR_NET_NOTINIT);
	}
}

vp_err_t
vp_net_close(int fds[2]) {

	close(fds[0]);
	close(fds[1]);
	return (VP_ERR_NONE);
}

vp_err_t
vp_net_talk(int fds[2]) {
	char *line;
	size_t sz;
	vp_err_t r;

	/* Handshake: expect the server to tell us who they are */
	if (r = vp_readline(fds[0], &line, &sz), r)
		return (r);
	if (line == NULL)
		return (VP_ERR_INT);
	RDBG(("Remote server greeting: %s", line));
	/* FIXME: examine that to determine POP3 or SMTP responses */
	free(line);

	if (r = vp_net_cmdresp2(fds, "USER", vp_user), r)
		return (r);
	if (r = vp_net_cmdresp2(fds, "PASS", vp_oldp), r)
		return (r);
	if (vp_cuser != NULL)
		if (r = vp_net_cmdresp2(fds, "CUSER", vp_cuser), r)
			return (r);
	if (r = vp_net_cmdresp2(fds, "NEWPASS", vp_newp), r)
		return (r);

	return (VP_ERR_NONE);
}

static vp_err_t
vp_net_cmdresp2(int fds[2], const char *cmd, const char *arg) {
	char *cmdstr, *resp;
	int err;
	vp_err_t r;

	cmdstr = resp = NULL;
	do {
		if (r = vp_strmcat(&cmdstr, 3, cmd, " ", arg), r)
			break;
		RDBG(("sending command '%s'\n", cmdstr));
		if (r = vp_net_sendexp(fds, cmdstr, &resp, &err), r)
			break;
		RDBG(("received response '%s', err %d\n", resp, err));
		if (err) {
			fprintf(stderr, "Sending '%s': %s\n", cmd, resp);
			r = VP_ERR_COMM;
		}
	} while (0);

	free(cmdstr);
	free(resp);
	return (r);
}

static vp_err_t
vp_net_sendexp(int fds[2], const char *s, char **resp, int *perr) {
	char *line;
	size_t sz;
	int err;
	vp_err_t r;

	if (r = vp_writeline(fds[1], s, 1), r)
		return (r);
	if (r = vp_readline(fds[0], &line, &sz), r)
		return (r);
	/* FIXME: fugly and incorrect for an SMTP-style dialogue */
	err = !!memcmp(line, "+OK", 3);

	*resp = line;
	*perr = err;
	return (VP_ERR_NONE);
}
