#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "vpass.h"
#include "vp_util.h"

static char	cvs_id[] __unused =
"$Id: vp_util.c,v 1.1 2001/08/21 12:33:16 roam Exp $";

#define VP_BUFSIZ	2048

vp_err_t
vp_strmcat(char **pdst, unsigned n, ...) {
	char *str, *new;
	const char *s;
	size_t len, ls;
	unsigned i;
	va_list v;

	str = NULL;
	len = 0;

	va_start(v, n);
	for (i = 0; i < n; i++) {
		s = va_arg(v, const char *);
		if (s == NULL)
			return (VP_ERR_INT);
		ls = strlen(s);
		if (new = realloc(str, len + ls + 1), new == NULL) {
			free(str);
			return (VP_ERR_NOMEM);
		}
		str = new;
		memcpy(str + len, s, ls);
		len += ls;
	}
	va_end(v);

	str[len] = '\0';
	*pdst = str;
	return (VP_ERR_NONE);
}

/*
 * Function:
 * 	vp_readline			- non-stdio-based line-by-line read
 * Inputs:
 * 	fd				- file descriptor to read from
 * 	pbuf				- val/res pointer for the line read
 * 	psz				- val/res pointer for the line length
 * Returns:
 * 	vp_err_t
 * 	NOMEM, READ
 * Modifies:
 * 	allocates and stores into *pbuf, *psz
 */

vp_err_t
vp_readline(int fd, char **pbuf, size_t *psz) {
	static char *buf = NULL;
	static size_t ofs = 0, all = 0;
	static int lasttry = 0;
	char *line, *nl;
	size_t len;
	ssize_t n;

	buf = line = NULL;
	all = ofs = 0;

	for (;;) {
		/* First check for an inherited newline */
		if (lasttry ||
		    ((ofs > 0) && (nl = memchr(buf, '\n', ofs), nl != NULL))) {
			if (lasttry) {
				if (ofs == NULL) {
					*pbuf = NULL;
					*psz = 0;
					return (VP_ERR_NONE);
				}
				nl = buf + ofs;
				len = ofs;
			} else {
				/* Found a newline!  Go for it! */
				len = nl - buf + 1;
			}

			if (line = calloc(1, len + 1), line == NULL)
				return (VP_ERR_NOMEM);
			memmove(line, buf, len);
			memmove(buf, buf + len, ofs - len);
			ofs -= len;

			*pbuf = line;
			*psz = len;
			return (VP_ERR_NONE);
		}

		/* Need to allocate? */
		if (ofs == all) {
			/* Allocate the new chunk */
			line = realloc(buf, all + VP_BUFSIZ);
			if (line == NULL)
				return (VP_ERR_NOMEM);
			buf = line;
			all += VP_BUFSIZ;
		}

		/* Read.. */
		if (n = read(fd, buf + ofs, all - ofs), n == -1) {
			free(buf);
			buf = NULL;
			return (VP_ERR_READ);
		} else if (n == 0) {
			lasttry = 1;
		}
		ofs += n;
	}
	/* NOTREACHED */
}

vp_err_t
vp_writeline(int fd, const char *line, int crlf) {
	ssize_t len, written, n;

	len = strlen(line);
	for (written = n = 0; written < len; written += n) {
		if (n = write(fd, line + written, len - written), n == -1)
			return (VP_ERR_WRITE);
		if (n == 0)
			return (VP_ERR_NONE);
	}
	if (crlf)
		return (vp_writeline(fd, "\r", 0));
	
	if (n = write(fd, "\n", 1), n == -1)
		return (VP_ERR_WRITE);
	return (VP_ERR_NONE);
}
